// ======================================================================================
//  Vehicle Info
//  Version: 2.5.0
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
// ======================================================================================
//
//  Gives turret and user action information for nearest vehicle or objects,
//  and gives the option to activate it via the action menu.
//
//  Place into your mission folder, and call via:
//    nul=[] execVM "vehinfo.sqf"
//
// ======================================================================================

#include "\vbs2\headers\function_library.hpp"

_sepline = "-------------------------------------------\n";
// legacy animations
_animations = [
	"aileron_1_engine",
	"aileron_2_engine",
	"anim_arm",
	"anim_arm_2",
	"anim_bay_door_1",
	"anim_bay_door_1_acutator",
	"anim_bay_door_2",
	"anim_bay_door_2_acutator",
	"arms_anim",
	"articulation_anim",
	"boom_anim",
	"boom_telescope_anim",
	"bucket_anim",
	"crane_port_arm_1",
	"crane_port_arm_2",
	"crane_port_arm_3",
	"crane_port_trav",
	"crane_stbd_arm_1",
	"crane_stbd_arm_2",
	"crane_stbd_arm_3",
	"crane_stbd_trav",
	"cranearm_1",
	"cranearm_2",
	"cranearmcylinder_1_pt1",
	"cranearmcylinder_1_pt2",
	"cranearmpiston_1",
	"cranelock",
	"cranetrav",
	"de_rack_port",
	"de_rack_stbd",
	"door_1",
	"door_1_slave",
	"door_2",
	"door_3",
	"door_4",
	"door_stern_anim",
	"dozerarm_1_pt1",
	"dozerarm_1_pt2",
	"dozerarm_2_pt1",
	"dozerarm_2_pt2",
	"dozerarm_2_pt3",
	"dozerarm_2_pt4",
	"dozerblade_pt1",
	"dozerblade_pt2",
	"dozerblade_pt3",
	"dozerblade_pt4",
	"dozercylinder_1_pt1",
	"dozercylinder_1_pt2",
	"dozercylinder_1_pt3",
	"dozercylinder_1_pt4",
	"dozerpiston_1_pt1",
	"dozerpiston_1_pt2",
	"dozerpiston_1_pt3",
	"dozerpiston_1_pt4",
	"elevator_1_engine",
	"elevator_2_engine",
	"flap_1",
	"flap_2_engine",
	"flap_3_engine",
	"flap_4",
	"front_blade_anim",
	"front_piston_anim",
	"front_piston_trans_anim",
	"gate_open",
	"hide_all",
	"hide_bogie",
	"hide_trailer",
	"hinge_anim",
	"hose_1",
	"hose_2",
	"launcher_1_pt1",
	"navlight1",
	"navlight2",
	"piston_anim",
	"railing_01_anim",
	"railing_06_anim",
	"railing_1",
	"ramp",
	"ramp_upper",
	"rear_ripper_anim",
	"rear_ripper_cylinder",
	"rear_ripper_hinge_1_anim",
	"rear_ripper_hinge_2_anim",
	"recoil",
	"recoil_1",
	"recoil_2",
	"side_hydrolic_anim",
	"strut_anim",
	"trailer_rotate"
];

if (count _this>1) then {
	_args = _this select 3;
	_vehicle = _args select 0;
	_action = _args select 1;
	switch (_action) do {
		case "movein": {
			// get rid of any crew
			{if !(isPlayer _x) then {unassignVehicle _x; _x setPos [0,0]; deleteVehicle _x}} forEach (crew _vehicle);
			_pos1 = _args select 2;
			_pos2 = _args select 3;
			if (_pos2==-1) then {
				player moveInTurret [_vehicle,[_pos1]];
				player sidechat format["player moveInTurret [_vehicle,%1]",_pos1];
			} else {
				player moveInTurret [_vehicle,[_pos1,_pos2]];
				player sidechat format["player moveInTurret [_vehicle,[%1,%2]]",_pos1,_pos2];
			};
		};
		case "action": {
			_action = _args select 2;
			_action = [_action,"_this","player"] call fn_vbs_replaceString;
			_action = [_action,"this","_vehicle"] call fn_vbs_replaceString;
			call compile _action;
			player sidechat _action;
		};
		case "animate": {
			_anim = _args select 2;
			_currphase = round (_vehicle animationPhase _anim);
			if (_currphase==.5) then {_currphase=1};
			_vehicle animate [_anim,1-_currphase];
			player sidechat format["_vehicle animate [""%1"",%2]",_anim,1-_currphase];
		};
	};

} else {

	_addAction = {
		_actionID1 = player addAction [_this select 1,"vehinfo.sqf",_this select 2,1,true,true,"","player==vehicle player"];
		_actionVeh = _this select 0;
		_actionID2 = _actionVeh addAction [_this select 1,"vehinfo.sqf",_this select 2,1,true,true,"","player!=vehicle player"];
		_actionIDs = _actionIDs + [[_actionID1,_actionVeh,_actionID2]];
	};

	_actionIDs= [];	
	_lstveh = "";
	_lastpos = getPos player;
	_refresh = 0;

	_classes = ["AllVehicles","Static"];
	
	while {true} do {
		_vehlist = nearestObjects [player,_classes,50];
		if (player in _vehlist) then {
			_vehlist = _vehlist - [player];
		};
		_veh = objNull;
		_anims = [];
		{
			_tstveh = _x;
			// does vehicle have animations?
			_anims = [_tstveh, player] call compile preprocessfile '\vbs2\plugins\iwa\data\scripts\findAnims.sqf';
			{
				if (_tstveh animationExists _x) then {
					_anims = _anims + [_x];
				};
			}forEach _animations;
			if (count _anims>0) exitWith {
				_veh = _tstveh;
			};
			// does vehicle have turrets?
			if (count (configFile >> "CfgVehicles" >> typeof _tstveh >> "turrets")>0) exitWith {
				_veh = _tstveh;
			};
			// does vehicle have actions?
			if (count (configFile >> "CfgVehicles" >> typeof _tstveh >> "UserActions")>0) exitWith {
				_veh = _tstveh;
			};
		}forEach _vehlist;
		_vehType = if !(isNull _veh) then {typeOf _veh} else {""};

		// a new vehicle was approached, get its parameters
		if ((_lstveh!=_vehType) && (_vehType!="") && !("stp" in animationState player)) then {
			currveh = _veh;
			clearRadio;

			{player removeAction (_x select 0); (_x select 1) removeAction (_x select 2)}forEach _actionIDs;
			_actionIDs = [];	

			_dn = getText (configFile >> "CfgVehicles" >> typeof _veh >> "displayName");
			_out = format["Vehicle: %1\n(%2)\n",typeof _veh,_dn];
				
			_lstveh = typeOf _veh;
			// find turrets
			_cfg = configFile >> "CfgVehicles" >> typeof _veh >> "turrets";
			_tc = count _cfg;
			if (_tc>0) then {
				_mtc = count _cfg; // number of main turrets
				_out = _out + format["%1Main Turrets: %2\n",_sepline,count _cfg];
				
				for "_mti" from 0 to _mtc-1 do {
					_mt = (_cfg select _mti);
					_st = _mt >> "turrets";
					_stc = count _st; // sub-turrets in current main one
					_out = _out + format["Turret #%1, %2: [%1]\n",_mti,configName(_mt)];
					_weaps = getArray(_mt >> "weapons");
					_out = _out + format[" Weapons:\n"];
					{_out = _out + format["  %1\n",_x]}forEach _weaps;
					[_veh,format["MoveIn: %1 [%2]",configName(_mt),_mti],[_veh,"movein",_mti,-1]] call _addAction;
					for "_sti" from 0 to _stc-1 do {
						_stp = (_st select _sti);
						_out = _out + format["Turret #%1, %2: [%1,%3]\n",_mti,configName(_stp),_sti];
						_weaps = getArray(_stp >> "weapons");
						_out = _out + format[" Weapons:\n"];
						{_out = _out + format["  %1\n",_x]}forEach _weaps;
						[_veh,format["MoveIn: %1 [%2,%3]",configName(_stp),_mti,_sti],[_veh,"movein",_mti,_sti]] call _addAction;
					};
				};
			};

			// find user actions
			_cfg = configFile >> "CfgVehicles" >> typeof _veh >> "UserActions";
			_ac = count _cfg;
			if (_ac>0) then {
				_mtc = count _cfg; // number of user actions
				_out = _out + format["%1User Actions: %2\n",_sepline,count _cfg];
				
				_neardist=9999;
				for "_mti" from 0 to _mtc-1 do {
					_mt = (_cfg select _mti);
					_st = _mt >> "UserActions";
					_selectionPos = _veh modelToWorld (_veh selectionPosition getText(_mt >> "position"));
					_dist = player distance _selectionPos;
					if (_dist<_neardist) then {
						_neardist=_dist;
					};
				};
				for "_mti" from 0 to _mtc-1 do {
					_mt = (_cfg select _mti);
					_st = _mt >> "UserActions";
					_stc = count _st; // available actions
					_statement = getText(_mt >> "statement");
					_displayname = getText(_mt >> "displayname");
					_selection = getText(_mt >> "position");
					_selectionPos = _veh modelToWorld (_veh selectionPosition _selection);
					_dist = player distance _selectionPos;
					if (abs(_dist-_neardist)<.5) then {
						_out = _out + format["%1: %2 [%3]\n",_mti+1,_displayname,_selection];
						[_veh,format["Action: %1",_displayname],[_veh,"action",_statement]] call _addAction;
					};
				};
			};			
			
			// find animations
			if (count _anims>0) then {
				_out = _out + format["%1Animations: %2\n",_sepline,count _anims];
				{
					_out = _out + format["  %1\n",_x];
					[_veh,format["Animate: %1",_x],[_veh,"animate",_x]] call _addAction;
				}forEach _anims;
			};
						
			hint _out;

			if !(_veh in _vehicles) then {
				_vehicles = _vehicles + [_veh];
			};
			
		} else {
			if (isNull _veh) then {
				hint "No vehicle with turrets or user actions in range.";
			};
		};
		sleep 1;
		_refresh=_refresh+1;
		if (_refresh>20) then {_refresh=0; _lstveh="xxx"};
	};
};